@@ -6,6 +6,7 @@ from .mumps import (
     schur_complement,
     nullspace,
     possible_orderings,
+    orderings,
 )
 from ._version import __version__
 
@@ -17,13 +18,17 @@ __all__ = [
     "schur_complement",
     "nullspace",
     "possible_orderings",
+    "orderings",
     "__version__",
     "test",
 ]
 
 
 def test(verbose=True):
-    from pytest import main
+    try:
+        from pytest import main
+    except ImportError:
+        raise RuntimeError("pytest is required to run the tests")
     import os.path
 
     return main(
